package com.simingtang.smt_face_ml_kit

import android.content.Context
import android.content.Intent
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import android.util.Log
import androidx.annotation.NonNull
import androidx.camera.core.CameraSelector
import androidx.camera.core.ImageAnalysis
import androidx.camera.core.ImageCapture
import androidx.camera.core.Preview
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.camera.view.PreviewView
import androidx.core.content.ContextCompat
import com.google.common.util.concurrent.ListenableFuture
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.util.*
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors
import kotlin.concurrent.timerTask

class SmtFaceMlKitPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    companion object {
        lateinit var channel: MethodChannel
        lateinit var plugin: SmtFaceMlKitPlugin
    }

    private lateinit var mFlutterPluginBinding: FlutterPlugin.FlutterPluginBinding
    private var cameraExecutor: ExecutorService? = null
    private var sensorManager: SensorManager? = null
    private var cameraProviderFuture: ListenableFuture<ProcessCameraProvider>? = null
    private var cameraProvider: ProcessCameraProvider? = null
    private lateinit var mBinding: ActivityPluginBinding
    private var smtFaceMlKitActivity: SmtFaceMlKitActivity? = null
    private lateinit var previewView: PreviewView


    fun activityStarted(activity: SmtFaceMlKitActivity) {
        smtFaceMlKitActivity = activity
        previewView = PreviewView(activity)

        previewView.setBackgroundColor(activity.resources.getColor(android.R.color.white))

        val factory = SMTCameraViewFactory(previewView)

        mFlutterPluginBinding.platformViewRegistry.registerViewFactory(
            "smt_camera_view",
            factory
        )
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        Log.e("flutter", "执行顺序2")
        plugin = this
        mFlutterPluginBinding = flutterPluginBinding
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "smt_face_ml_kit")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "startFaceMlKit" -> {
                if(!::previewView.isInitialized){
                    result.error("-1","初始化未完成","")
                    return
                }
                initSensor()
                startCamera()
            }
            "endFaceMlKit" -> {
                finishSmtFaceMlKitActivity()
            }
        }
        result.success(null)
    }

    private fun finishSmtFaceMlKitActivity() {
        sensorManager?.unregisterListener(listener)
        cameraExecutor?.shutdown()
        cameraProvider?.unbindAll()
        cameraProviderFuture?.cancel(true)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        finishSmtFaceMlKitActivity()
    }

    //第三步：对传感器信号进行监听
    private val listener: SensorEventListener = object : SensorEventListener {
        override fun onSensorChanged(event: SensorEvent) {
            channel.invokeMethod("lightInfo", event.values[0].toString())
        }

        override fun onAccuracyChanged(sensor: Sensor, accuracy: Int) {}
    }

    private fun initSensor() {
        //第一步：获取 SensorManager 的实例
        sensorManager =
            smtFaceMlKitActivity?.getSystemService(Context.SENSOR_SERVICE) as SensorManager
        //第二步：获取 Sensor 传感器类型
        val sensor: Sensor = sensorManager!!.getDefaultSensor(Sensor.TYPE_LIGHT)
        //第四步：注册 SensorEventListener
        sensorManager!!.registerListener(listener, sensor, SensorManager.SENSOR_DELAY_NORMAL)
    }

    private fun startCamera() {
        cameraExecutor = Executors.newSingleThreadExecutor()

        cameraProviderFuture = ProcessCameraProvider.getInstance(smtFaceMlKitActivity!!)
        cameraProviderFuture!!.addListener({
            cameraProvider = cameraProviderFuture!!.get()

            val imageCapture = ImageCapture.Builder()
                .build()

            val preview = Preview.Builder()
                .build()
                .also {
                    it.setSurfaceProvider(previewView.surfaceProvider)
                }

            val imageAnalyzer = ImageAnalysis.Builder()
                .build()
                .also {
                    it.setAnalyzer(cameraExecutor!!, FaceAnalyzer())
                }

            try {
                cameraProvider!!.unbindAll()

                cameraProvider!!.bindToLifecycle(
                    smtFaceMlKitActivity!!,
                    CameraSelector.DEFAULT_FRONT_CAMERA,
                    preview,
                    imageCapture,
                    imageAnalyzer
                )

            } catch (exc: Exception) {
                Log.e("flutter", exc.toString())
                channel.invokeMethod("error", "错误信息：相机绑定信息出错")
            }

        }, ContextCompat.getMainExecutor(smtFaceMlKitActivity!!))
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {

        mBinding = binding

        val intent = Intent(mBinding.activity, SmtFaceMlKitActivity::class.java)
        mBinding.activity.startActivity(intent)

    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
    }
}
