import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/services.dart';
import 'package:permission_handler/permission_handler.dart';

typedef FacesList = void Function(List faces);

typedef LightValue = void Function(String value);

typedef FaceImage = void Function(Uint8List bytes);

typedef ErrorMsg = void Function(String error);

class SmtFaceMlKit {
  static FacesList? _facesList;
  static LightValue? _lightValue;
  static ErrorMsg? _errorMsg;

  static final MethodChannel _channel = const MethodChannel('smt_face_ml_kit')
    ..setMethodCallHandler((call) async {
      switch (call.method) {
        case "lightInfo":
          _lightValue?.call(call.arguments);
          break;
        case "faceInfos":
          _facesList?.call(call.arguments);
          break;
        case "error":
          _errorMsg?.call(call.arguments);
          break;
      }
    });

  static Future startFaceMlKit(
      {required FacesList facesList,
      LightValue? lightValue,
      ErrorMsg? errorMsg}) async {
    var status = await Permission.camera.request();
    switch (status) {
      case PermissionStatus.denied:
        _errorMsg?.call("权限拒绝");
        return;
      case PermissionStatus.granted:
        break;
      case PermissionStatus.permanentlyDenied:
        _errorMsg?.call("权限拒绝，请手动打开相机权限");
        openAppSettings();
        return;
      case PermissionStatus.limited:
        break;
      case PermissionStatus.restricted:
        break;
      default:
        break;
    }

    _facesList = facesList;
    _lightValue = lightValue;
    _errorMsg = errorMsg;
    await _channel.invokeMethod('startFaceMlKit');
  }

  static Future endFaceMlKit() async {
    _channel.invokeMethod('endFaceMlKit');
  }
}
